package cn.qingshi.gamesdk.core.network;

import android.content.Context;
import android.text.TextUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import cn.qingshi.gamesdk.base.utils.Logger;
import cn.qingshi.gamesdk.core.entity.ResultInfo;
import cn.qingshi.gamesdk.core.internal.IFileRequestCallback;
import cn.qingshi.gamesdk.core.internal.IRequestCallback;
import cn.yyxx.support.FileUtils;
import cn.yyxx.support.JsonUtils;
import cn.yyxx.support.volley.VolleySingleton;
import cn.yyxx.support.volley.source.DefaultRetryPolicy;
import cn.yyxx.support.volley.source.NetworkResponse;
import cn.yyxx.support.volley.source.ParseError;
import cn.yyxx.support.volley.source.Request;
import cn.yyxx.support.volley.source.Response;
import cn.yyxx.support.volley.source.VolleyError;
import cn.yyxx.support.volley.source.toolbox.HttpHeaderParser;
import cn.yyxx.support.volley.source.toolbox.JsonObjectRequest;
import cn.yyxx.support.volley.source.toolbox.JsonRequest;

/**
 * @author #Suyghur.
 * Created on 2021/04/28
 */
public class VolleyRequest {

    /**
     * 超时时间
     */
    private static final int MAX_TIMEOUT = 10 * 1000;


    public static void post(Context context, String url, JSONObject jsonObject, final IRequestCallback callback) {
        try {
//            StringBuilder raw = new StringBuilder();
//            raw.append("access_token=").append(jsonObject.getString("access_token"))
//                    .append("&time=").append(jsonObject.getString("time"))
//                    .append("&context=").append(jsonObject.getString("context"))
//                    .append("&appkey=").append(ParamsUtils.getQSAppKey(context));
//
//            Logger.d("raw : " + raw);
//            String sign = Md5Utils.encodeByMD5(raw.toString().toLowerCase());
//            jsonObject.put("sign", sign);
            Logger.logHandler("请求地址 : " + url + "\n");
            Logger.logHandler("请求参数 : " + jsonObject.toString() + "\n");
            Logger.d("请求地址 : " + url);
            Logger.d("请求参数 : " + jsonObject.toString());

            JsonRequest<JSONObject> request = new JsonObjectRequest(Request.Method.POST, url, jsonObject, new Response.Listener<JSONObject>() {
                @Override
                public void onResponse(JSONObject response) {
                    ResultInfo resultInfo = new ResultInfo();
                    if (response != null) {
                        try {
                            resultInfo.code = response.getInt("code");
                            resultInfo.msg = response.getString("message");
                            if (JsonUtils.hasJsonKey(response, "data")) {
                                resultInfo.data = response.getString("data");
                            } else {
                                resultInfo.data = "";
                            }
                        } catch (JSONException e) {
                            e.printStackTrace();
                            resultInfo.code = -1;
                            resultInfo.msg = "数据解析异常";
                            resultInfo.data = "";
                        }
                    } else {
                        resultInfo.code = -1;
                        resultInfo.msg = "接口请求异常";
                        resultInfo.data = "";
                    }
                    Logger.logHandler("返回内容 : " + resultInfo.toString());
                    Logger.d("onResponse : " + resultInfo.toString());
                    if (callback != null) {
                        callback.onResponse(resultInfo);
                    }
                }
            }, new Response.ErrorListener() {
                @Override
                public void onErrorResponse(VolleyError volleyError) {
                    Logger.e("onErrorResponse : " + volleyError.toString());
                    if (callback != null) {
                        callback.onResponse(getErrorResultInfo(volleyError));
                    }
                }
            }) {
                @Override
                public Map<String, String> getHeaders() {
                    HashMap<String, String> headers = new HashMap<>();
                    headers.put("Accept", "application/json");
                    headers.put("Content-Type", "application/json;charset=UTF-8");
                    return headers;
                }
            };
            request.setRetryPolicy(new DefaultRetryPolicy(MAX_TIMEOUT, 1, 1.0f));
            VolleySingleton.getInstance(context.getApplicationContext()).addToRequestQueue(context.getApplicationContext(), request);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void downloadImageFile(Context context, String url, final String filePath, final IFileRequestCallback callback) {
        if (TextUtils.isEmpty(filePath)) {
            callback.onResult(-1, "file path is empty !!!");
            return;
        }
        Request<byte[]> request = new Request<byte[]>(Request.Method.GET, url, new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError volleyError) {
                callback.onResult(-1, volleyError.getLocalizedMessage());
            }
        }) {

            @Override
            protected Response<byte[]> parseNetworkResponse(NetworkResponse networkResponse) {
                try {
                    if (networkResponse.data == null) {
                        return Response.error(new ParseError(networkResponse));
                    } else {
                        return Response.success(networkResponse.data, HttpHeaderParser.parseCacheHeaders(networkResponse));
                    }
                } catch (OutOfMemoryError e) {
                    e.printStackTrace();
                    return Response.error(new ParseError(e));
                }
            }

            @Override
            protected void deliverResponse(byte[] bytes) {
                Logger.d("volley download image file success, start to save file ...");
                try {
                    FileUtils.saveFile(filePath, bytes);
                    callback.onResult(0, "缓存文件成功");
                } catch (IOException e) {
                    e.printStackTrace();
                    callback.onResult(0, "缓存文件失败");
                }
            }
        };
        VolleySingleton.getInstance(context.getApplicationContext()).addToRequestQueue(context.getApplicationContext(), request);
    }

    private static ResultInfo getErrorResultInfo(VolleyError volleyError) {
        ResultInfo resultInfo = new ResultInfo();
        if (volleyError.networkResponse != null) {
            resultInfo.code = volleyError.networkResponse.statusCode;
            resultInfo.msg = volleyError.getMessage();
        } else {
            resultInfo.code = 400;
            resultInfo.msg = "网络异常";
        }
        resultInfo.data = "";
        return resultInfo;
    }
}
